#!/usr/local/bin/perl
use strict;

# @author: remy sharp / http://remysharp.com
# @date: 2007-07-02
# @description: displays a list (in JSON for now) of people you haven't been 
# in touch for in while.
# @license: Creative Commons License - ShareAlike http://creativecommons.org/licenses/by-sa/3.0/

# requires
# http://search.cpan.org/~timb/DBI-1.58/DBI.pm
# http://search.cpan.org/~msergeant/DBD-SQLite-1.13/lib/DBD/SQLite.pm

# usage:
# Run once to pull up a list of all the mail boxes.  I recommend you run as such to filter:
#   perl email_catchup.pl mailboxes | grep -i sent
# Then copy each mail box URL in to the @mailboxes array below.
#
# Then add a list of your contact's email addresses in the @friends array.
# Finally, choose your cut off period - the format can be:
#   1 day ago
#   3 months ago ... etc.

# Edit these variables
my @mailboxes = ();

# add a list of your friend's email addresses here
# e.g. my @friends = ( 'john@doe.com', 'bill@ben.com' );
my @friends = ();

my $too_long_ago = date_ago('5 days ago');

use Data::Dumper qw(Dumper);
if ($#ARGV > 1) {
	@mailboxes = split(',', shift @ARGV);
	@friends = split(',', shift @ARGV);
	$too_long_ago = date_ago(join(' ', @ARGV));
}

# [try] not to edit below here
use DBI;
use POSIX qw(floor);
use Time::Local;

main();

sub main {

	my ($sql, $i, $rows);
	my $dbname = "/Users/$ENV{'USER'}/Library/Mail/Envelope Index";
	
	my $dbh = DBI->connect("dbi:SQLite:dbname=" . $dbname,"","") || die "Cannot connect: $DBI::errstr";
	
	if (scalar(@mailboxes) == 0 || $ARGV[0] eq 'mailboxes') {
		($i, $rows) = query($dbh, 'select * from mailboxes');
		my @out = ();
		foreach (@$rows) {
			push(@out, qq{"$_->{'url'}"}) if ($_->{'url'} =~ /sent/i);
		}

		print "[" . join(',', @out) . "]\n";
	} else {
		# get the mail box ids
		my @mb_ids = ();
		foreach my $mb (@mailboxes) {
			($i, $rows) = query($dbh, 'select * from mailboxes where url="' . $mb . '"');
			push(@mb_ids, $rows->[0]->{'ROWID'});
		}

		my $mailbox_ids = join(',', @mb_ids);

		# find address ids for friends
		my @friend_ids = ();
		foreach my $friend (@friends) {
			($i, $rows) = query($dbh, 'select * from addresses where address like "' . $friend . '"');
			foreach (@$rows) {
				push(@friend_ids, {ROWID => $_->{'ROWID'}, address => $_->{'address'}});
			}
		}

		my $last_comms = {};
		foreach (@friend_ids) {
			my $addr = lc($_->{'address'});
			($i, $rows) = query($dbh, 'select max(m.date_sent) as maxds from recipients r, messages m where r.address_id=' . $_->{'ROWID'} . ' and r.message_id=m.ROWID and m.mailbox in (' . $mailbox_ids . ')');
			
			my $max_date_sent = $rows->[0]->{'maxds'};
			if ($i && $max_date_sent) {
				if (!$last_comms->{$addr}) {
					$last_comms->{$addr} = $max_date_sent;
				} elsif ($max_date_sent > $last_comms->{$addr}) {
					$last_comms->{$addr} = $max_date_sent;
				}			
			}
		}

		my @out = ();
		foreach (sort {$last_comms->{$b} <=> $last_comms->{$a}} keys %$last_comms) {		
			if ($last_comms->{$_} < $too_long_ago) {
				push(@out, '{ "who" : "' . $_ . '", "raw" : "' . $last_comms->{$_} . '", "date" : "' . localtime($last_comms->{$_}) . '", "nice" : "' . ago($last_comms->{$_}) . '"}');
			}		
		}

		print "[" . join(',', @out) . "]\n";	
	}
		
	$dbh->disconnect();	
}

sub date_ago {
	my ($t) = @_;
	$t =~ s/\s+ago.*$//;
	my ($n, $type) = split(' ', $t);
	
	my $now = time;
	
	my $cutoff = time;
	
	if ($type =~ /min/) {
		$cutoff -= ($n*60);
	} elsif ($type =~ /hour/) {
		$cutoff -= ($n*60*60);
	} elsif ($type =~ /day/) {
		$cutoff -= ($n*60*60*24);
	} elsif ($type =~ /mon/) {
		my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime($cutoff);
		
		while (($mon -= $n) < 0) {
			$year--;
			$mon = 12 + $mon;
			$n -= 12;
		}
		$cutoff = timelocal($sec,$min,$hour,$mday,$mon,$year);
	} elsif ($type =~ /year/) {
		my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime($cutoff);
		$year -= $n;
		$cutoff = timelocal($sec,$min,$hour,$mday,$mon,$year);
	}
	
	return $cutoff;
}


sub query {
	my ($dbh, $sql) = @_;
	
	#print "query: $sql\n";
	
    my $statement = $dbh->prepare($sql);
    $statement->execute;

	my @data = ();

	while (my $row = $statement->fetchrow_hashref) {
		push(@data, $row);
	}
	
	$statement->finish();
	
	return (scalar(@data), \@data);
}

sub pad {
	my $num = shift;
	return length($num) == 1 ? '0' . $num : $num;
}

sub ago {
	my $date = shift;

	
	my $Mod = $date; #/86400000;
	my $now = time(); #/86400000;

	my $daysago = ($now - $Mod)/60/60/24;

	my $unit = '';
	my $towrite = '';

	if ($daysago > 730)  {
		$daysago = floor($daysago/365);
		$unit = 'years';
	} elsif ($daysago > 60) {
		$daysago = floor($daysago/30);
		$unit = 'months';
	} elsif ($daysago > 14) {
		$daysago = floor($daysago/7);
		$unit = 'weeks';
	} elsif ($daysago > 2) {
		$daysago = floor($daysago);
		$unit = 'days';
	} elsif ($daysago > 1) {
		$daysago = floor($daysago);
		$unit = 'day';
	} elsif (($daysago * 24) > 1) {
		$daysago = floor($daysago * 24);
		$unit = 'hours';
	} elsif (($daysago * 24 * 60) > 1) {
		$daysago = floor($daysago * 24 * 60);
		$unit = 'minutes';
	} elsif (($daysago * 24 * 60 * 60) < 1) {
		$daysago = floor($daysago * 24 * 60 * 60);
		$unit = 'seconds'; # or 'just now!'
	}

	if ($daysago == 1) {
		$towrite .= 'yesterday';
	} else {
		$towrite .= $daysago . ' ' . $unit . ' ago';
	}
	
	return $towrite;
}